@@ -393,10 +393,10 @@ static char *process_path(const char *path, bool resolve_symlinks)
                    we want to be able to operate on broken symlinks. */
                 return strdup(path);
             }
-        } else if (strncmp(result, settings.mntdest, settings.mntdest_len) == 0) {
+        } else if (path_starts_with(result, settings.mntdest, settings.mntdest_len)) {
             /* Recursive call. We cannot handle this without deadlocking,
                especially in single-threaded mode. */
-            DPRINTF("Denying recursive access to mountpoint `%s'", result);
+            DPRINTF("Denying recursive access to mountpoint \"%s\" at \"%s\"", settings.mntdest, result);
             free(result);
             errno = EPERM;
             return NULL;
@@ -457,25 +457,26 @@ static int getattr_common(const char *procpath, struct stat *stbuf)
     if (settings.block_devices_as_files && S_ISBLK(stbuf->st_mode)) {
         stbuf->st_mode ^= S_IFBLK | S_IFREG;  // Flip both bits
         int fd = open(procpath, O_RDONLY);
+        if (fd == -1) {
+            return -errno;
+        }
 #ifdef __linux__
         uint64_t size;
         ioctl(fd, BLKGETSIZE64, &size);
         stbuf->st_size = (off_t)size;
         if (stbuf->st_size < 0) {  // Underflow
+            close(fd);
             return -EOVERFLOW;
         }
 #else
-        if (fd == -1) {
-            return -errno;
-        }
         off_t size = lseek(fd, 0, SEEK_END);
         if (size == (off_t)-1) {
             close(fd);
             return -errno;
         }
         stbuf->st_size = size;
-        close(fd);
 #endif
+        close(fd);
     }
 
     /* Then permission bits. Symlink permissions don't matter, though. */
@@ -709,7 +710,11 @@ static void *bindfs_init()
             strerror(errno)
             );
         bindfs_init_failed = true;
+#ifdef __OpenBSD__
+        exit(1);
+#else
         fuse_exit(fuse_get_context()->fuse);
+#endif
     }
 
     return NULL;
@@ -1663,7 +1668,9 @@ static struct fuse_operations bindfs_oper = {
     .lock       = bindfs_lock,
     .flock      = bindfs_flock,
 #endif
+#ifndef __OpenBSD__
     .ioctl      = bindfs_ioctl,
+#endif
     .statfs     = bindfs_statfs,
     .release    = bindfs_release,
     .fsync      = bindfs_fsync,
@@ -1801,6 +1808,7 @@ enum OptionKey {
     OPTKEY_HIDE_HARD_LINKS,
     OPTKEY_RESOLVE_SYMLINKS,
     OPTKEY_BLOCK_DEVICES_AS_FILES,
+    OPTKEY_DIRECT_IO,
     OPTKEY_NO_DIRECT_IO
 };
 
@@ -1910,6 +1918,9 @@ static int process_option(void *data, const char *arg, int key,
         settings.block_devices_as_files = 1;
         return 0;
 #ifdef __linux__
+    case OPTKEY_DIRECT_IO:
+        settings.direct_io = true;
+        return 0;
     case OPTKEY_NO_DIRECT_IO:
         settings.direct_io = false;
         return 0;
@@ -2408,6 +2419,7 @@ int main(int argc, char *argv[])
         OPT2("--delete-deny", "delete-deny", OPTKEY_DELETE_DENY),
         OPT2("--rename-deny", "rename-deny", OPTKEY_RENAME_DENY),
 #ifdef __linux__
+        OPT2("--direct-io", "direct-io", OPTKEY_DIRECT_IO),
         OPT2("--no-direct-io", "no-direct-io", OPTKEY_NO_DIRECT_IO),
 #endif
 
@@ -2477,7 +2489,7 @@ int main(int argc, char *argv[])
 #ifdef __linux__
     settings.forward_odirect = 0;
     settings.odirect_alignment = 0;
-    settings.direct_io = true;
+    settings.direct_io = false;
 #endif
 
     atexit(&atexit_func);
@@ -2820,10 +2832,17 @@ int main(int argc, char *argv[])
     }
 #endif
 
+#ifdef __OpenBSD__
+    if (settings.enable_ioctl) {
+        fprintf(stderr, "--enable-ioctl not supported on OpenBSD\n");
+        return 1;
+    }
+#else
     /* Remove the ioctl implementation unless the user has enabled it */
     if (!settings.enable_ioctl) {
         bindfs_oper.ioctl = NULL;
     }
+#endif
 
     /* Remove/Ignore some special -o options */
     args = filter_special_opts(&args);
