@@ -7,7 +7,7 @@
 
 extern int failures;
 
-#define TEST_ASSERT(expr) do { if (!(expr)) { printf("Assertion failed: %s:%d: `%s'\n", __FILE__, __LINE__, #expr); failures++; } } while (0);
+#define TEST_ASSERT(expr) do { if (!(expr)) { printf("Assertion failed: %s:%d: `%s'\n", __FILE__, __LINE__, #expr); ++failures; } } while (0);
 #define NEAR(a, b, eps) (fabs((a) - (b)) < (eps))
 
 int run_suite(void (*suite)());
